@@ -1,26 +1,31 @@
 package edu.berkeley.boinc.client;
 
-import java.util.ArrayList;
-
-import edu.berkeley.boinc.BOINCActivity;
-import edu.berkeley.boinc.R;
-import edu.berkeley.boinc.rpc.Result;
-import edu.berkeley.boinc.utils.Logging;
-
 import android.annotation.SuppressLint;
 import android.app.Notification;
 import android.app.NotificationManager;
 import android.app.PendingIntent;
 import android.content.Context;
 import android.content.Intent;
-import android.graphics.BitmapFactory;
-import android.support.v4.app.NotificationCompat;
+import android.os.Build;
 import android.util.Log;
 
+import androidx.core.app.NotificationCompat;
+import androidx.core.content.ContextCompat;
+
+import java.util.ArrayList;
+import java.util.List;
+
+import javax.inject.Inject;
+import javax.inject.Singleton;
+
+import edu.berkeley.boinc.BOINCActivity;
+import edu.berkeley.boinc.R;
+import edu.berkeley.boinc.rpc.Result;
+import edu.berkeley.boinc.utils.BOINCUtils;
+import edu.berkeley.boinc.utils.Logging;
+
+@Singleton
 public class ClientNotification {
-
-    private static ClientNotification clientNotification = null;
-
     private Context context;
     private NotificationManager nm;
     private Integer notificationId;
@@ -29,28 +34,16 @@ public class ClientNotification {
 
     private int mOldComputingStatus = -1;
     private int mOldSuspendReason = -1;
-    private ArrayList<Result> mOldActiveTasks = new ArrayList<>();
+    private List<Result> mOldActiveTasks = new ArrayList<>();
     private boolean notificationShown = false;
     // debug foreground state by running
     // adb shell: dumpsys activity services edu.berkeley.boinc
     private boolean foreground = false;
 
-    /**
-     * Returns a reference to a singleton ClientNotification object.
-     * Constructs a new instance of the ClientNotification if not already constructed.
-     *
-     * @return ClientNotification static instance
-     */
-    public static ClientNotification getInstance(Context ctx) {
-        if(clientNotification == null) {
-            clientNotification = new ClientNotification(ctx);
-        }
-        return clientNotification;
-    }
-
-    public ClientNotification(Context ctx) {
-        this.context = ctx;
-        this.nm = (NotificationManager) context.getSystemService(Context.NOTIFICATION_SERVICE);
+    @Inject
+    public ClientNotification(Context context) {
+        this.context = context;
+        this.nm = ContextCompat.getSystemService(context, NotificationManager.class);
         notificationId = context.getResources().getInteger(R.integer.autostart_notification_id);
         Intent intent = new Intent(context, BOINCActivity.class);
         intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
@@ -59,31 +52,33 @@ public class ClientNotification {
     }
 
     /**
-     * Updates notification with client's current status. Notifies if not present. Checking notification related preferences.
+     * Updates notification with client's current status. Notifies if not present. Checking
+     * notification-related preferences.
      *
      * @param updatedStatus client status data
      * @param service       reference to service, sets to foreground if active
-     * @param active        indicator whether BOINC should stay in foreground (during computing and idle, i.e. not suspended)
+     * @param active        indicator whether BOINC should stay in foreground (during computing and
+     *                      idle, i.e. not suspended)
      */
-    public void update(ClientStatus updatedStatus, Monitor service, Boolean active) {
-
+    public void update(ClientStatus updatedStatus, Monitor service, boolean active) {
         // nop if data is not present
         if(service == null || updatedStatus == null) {
             return;
         }
 
         //check if active tasks have changed to force update
-        Boolean activeTasksChanged = false;
+        boolean activeTasksChanged = false;
         if(active && updatedStatus.computingStatus == ClientStatus.COMPUTING_STATUS_COMPUTING) {
-            ArrayList<Result> activeTasks = updatedStatus.getExecutingTasks();
+            List<Result> activeTasks = updatedStatus.getExecutingTasks();
             if(activeTasks.size() != mOldActiveTasks.size()) {
                 activeTasksChanged = true;
             }
             else {
                 for(int x = 0; x < activeTasks.size(); x++) {
-                    if(!activeTasks.get(x).name.equals(mOldActiveTasks.get(x).name)) {
+                    if(!activeTasks.get(x).getName().equals(mOldActiveTasks.get(x).getName())) {
                         activeTasksChanged = true;
-                        Log.d("blub", "bla " + activeTasks.get(x).name + " vs. " + mOldActiveTasks.get(x).name); //TODO
+                        Log.d(Logging.TAG, "Active task: " + activeTasks.get(x).getName()
+                                           + ", old active task: " + mOldActiveTasks.get(x).getName());
                         break;
                     }
                 }
@@ -97,24 +92,23 @@ public class ClientNotification {
             activeTasksChanged = true;
         }
 
-        // update notification, only
-        // if it hasn't been shown before, or
-        // after change in status
+        // update notification, only if it hasn't been shown before, or after change in status
         if(Logging.VERBOSE) {
             Log.d(Logging.TAG,
-                  "ClientNotification: notification needs update? " + (clientNotification.mOldComputingStatus == -1)
+                  "ClientNotification: notification needs update? "
+                  + (mOldComputingStatus == -1)
                   + activeTasksChanged
                   + !notificationShown
-                  + (updatedStatus.computingStatus != clientNotification.mOldComputingStatus)
+                  + (updatedStatus.computingStatus != mOldComputingStatus)
                   + (updatedStatus.computingStatus == ClientStatus.COMPUTING_STATUS_SUSPENDED
-                     && updatedStatus.computingSuspendReason != clientNotification.mOldSuspendReason));
+                     && updatedStatus.computingSuspendReason != mOldSuspendReason));
         }
-        if(clientNotification.mOldComputingStatus == -1
+        if(mOldComputingStatus == -1
            || activeTasksChanged
            || !notificationShown
-           || updatedStatus.computingStatus != clientNotification.mOldComputingStatus
+           || updatedStatus.computingStatus != mOldComputingStatus
            || (updatedStatus.computingStatus == ClientStatus.COMPUTING_STATUS_SUSPENDED
-               && updatedStatus.computingSuspendReason != clientNotification.mOldSuspendReason)) {
+               && updatedStatus.computingSuspendReason != mOldSuspendReason)) {
 
             // update, build and notify
             nm.notify(notificationId, buildNotification(updatedStatus, active, mOldActiveTasks));
@@ -124,38 +118,28 @@ public class ClientNotification {
             notificationShown = true;
 
             // save status for comparison next time
-            clientNotification.mOldComputingStatus = updatedStatus.computingStatus;
-            clientNotification.mOldSuspendReason = updatedStatus.computingSuspendReason;
+            mOldComputingStatus = updatedStatus.computingStatus;
+            mOldSuspendReason = updatedStatus.computingSuspendReason;
         }
 
         // start foreground service, if requested
         // notification instance exists now, but might be out-dated (if screen is off)
         if(active && !foreground) {
-            setForegroundState(service, true);
+            setForegroundState(service);
         }
     }
 
     // Notification must be built, before setting service to foreground!
-    private void setForegroundState(Monitor service, Boolean foregroundState) {
-        if(foregroundState) {
-            service.startForeground(notificationId, n);
-            if(Logging.DEBUG) {
-                Log.d(Logging.TAG, "ClientNotification.setForeground() start service as foreground.");
-            }
-            foreground = true;
-        }
-        else {
-            foreground = false;
-            service.stopForeground(true);
-            notificationShown = false;
-            if(Logging.DEBUG) {
-                Log.d(Logging.TAG, "ClientNotification.setForeground() stop service as foreground.");
-            }
+    private void setForegroundState(Monitor service) {
+        service.startForeground(notificationId, n);
+        if(Logging.DEBUG) {
+            Log.d(Logging.TAG, "ClientNotification.setForeground() start service as foreground.");
         }
+        foreground = true;
     }
 
     @SuppressLint("InlinedApi")
-    private Notification buildNotification(ClientStatus status, Boolean active, ArrayList<Result> activeTasks) {
+    private Notification buildNotification(ClientStatus status, Boolean active, List<Result> activeTasks) {
         // get current client computingstatus
         Integer computingStatus = status.computingStatus;
         // get status strings from ClientStatus
@@ -165,8 +149,8 @@ public class ClientNotification {
         // build notification
         NotificationCompat.Builder nb = new NotificationCompat.Builder(context, "main-channel");
         nb.setContentTitle(statusTitle)
-          .setSmallIcon(getIcon(computingStatus))
-          .setLargeIcon(BitmapFactory.decodeResource(context.getResources(), getIcon(computingStatus)))
+          .setSmallIcon(getIcon(computingStatus, true))
+          .setLargeIcon(BOINCUtils.getBitmapFromVectorDrawable(context, getIcon(computingStatus, false)))
           .setContentIntent(contentIntent);
 
         // adapt priority based on computing status
@@ -181,11 +165,15 @@ public class ClientNotification {
         // set action based on computing status
         if(computingStatus == ClientStatus.COMPUTING_STATUS_NEVER) {
             // add resume button
-            nb.addAction(R.drawable.playw, context.getString(R.string.menu_run_mode_enable), getActionIntent(2));
+            // 0 - only text. Unify all versions of android with text button.
+            nb.addAction(0,
+                         context.getString(R.string.menu_run_mode_enable), getActionIntent(2));
         }
         else {
             // add suspend button
-            nb.addAction(R.drawable.pausew, context.getString(R.string.menu_run_mode_disable), getActionIntent(1));
+            // 0 - only text. Unify all versions of android with text button.
+            nb.addAction(0,
+                         context.getString(R.string.menu_run_mode_disable), getActionIntent(1));
         }
 
         // set tasks if computing
@@ -197,7 +185,7 @@ public class ClientNotification {
             // set names in list
             NotificationCompat.InboxStyle inboxStyle = new NotificationCompat.InboxStyle();
             for(Result task : activeTasks) {
-                inboxStyle.addLine(task.project.getName() + ": " + task.app.getName());
+                inboxStyle.addLine(task.getProject().getName() + ": " + task.getApp().getDisplayName());
             }
             nb.setStyle(inboxStyle);
         }
@@ -218,23 +206,25 @@ public class ClientNotification {
     }
 
     // returns resource id of icon
-    private int getIcon(int status) {
+    private int getIcon(int status, boolean isSmall) {
         int icon;
         switch(status) {
             case ClientStatus.COMPUTING_STATUS_NEVER:
-                icon = R.drawable.ic_stat_notify_boinc_paused;
-                break;
             case ClientStatus.COMPUTING_STATUS_SUSPENDED:
-                icon = R.drawable.ic_stat_notify_boinc_paused;
-                break;
             case ClientStatus.COMPUTING_STATUS_IDLE:
-                icon = R.drawable.ic_stat_notify_boinc_paused;
-                break;
-            case ClientStatus.COMPUTING_STATUS_COMPUTING:
-                icon = R.drawable.ic_stat_notify_boinc_normal;
+                if (Build.VERSION.SDK_INT < Build.VERSION_CODES.LOLLIPOP && isSmall) {
+                    icon = R.mipmap.ic_boinc_paused_white;
+                }
+                else {
+                    icon = R.drawable.ic_boinc_paused;
+                }
                 break;
             default:
-                icon = R.drawable.ic_stat_notify_boinc_normal;
+                if (Build.VERSION.SDK_INT < Build.VERSION_CODES.LOLLIPOP && isSmall) {
+                    icon = R.mipmap.ic_boinc_white;
+                } else {
+                    icon = R.drawable.ic_boinc;
+                }
         }
         return icon;
     }
